#include "2017_3_h.h"
#include "mbed.h"
#include <math.h>

Omni_3::Omni_3(PinName pin_pwm_F, PinName pin_dere_F, PinName pin_channelA_F, PinName pin_channelB_F,
               PinName pin_pwm_L, PinName pin_dere_L, PinName pin_channelA_L, PinName pin_channelB_L,
               PinName pin_pwm_R, PinName pin_dere_R, PinName pin_channelA_R, PinName pin_channelB_R,
               int arg_rev)
{
    md_f = Create_MD_PID(pin_channelA_F, pin_channelB_F, NC, 500, QEI::X4_ENCODING,
                         0.095 * 0.125, 0, 0.000125 * 4, 500,
                         pin_pwm_F, pin_dere_F);

    md_l = Create_MD_PID(pin_channelA_L, pin_channelB_L, NC, 500, QEI::X4_ENCODING,
    					 0.095 * 0.125, 0, 0.000125 * 4, 500,
    					 pin_pwm_L, pin_dere_L);

    md_r = Create_MD_PID(pin_channelA_R, pin_channelB_R, NC, 500, QEI::X4_ENCODING,
    					 0.095 * 0.125, 0, 0.000125 * 4, 500,
    					 pin_pwm_R, pin_dere_R);

    rev = arg_rev;
}

void Omni_3::Drive( int arg_a, double arg_x, double arg_y, double arg_rota)
{
    double dere, sp;
    double duty[3];
    double speed[3];

    dere = atan2(arg_y, arg_x);
    sp = pow(arg_x * arg_x + arg_y * arg_y, 0.60);

    speed[0] = cos(dere) * sp;   // vx
    speed[1] = sin(dere) * sp;   // vy
    speed[2] = arg_rota;         // omega

    Matrix(speed, duty);
    for(int i = 0; i < 3; i++){
        if(fabs(duty[i]) > 1.0){
            int inv = fabs(duty[i]);
            for(int j = 0; j < 3; j++){
                duty[j] *= 1.0 / (double)inv;
            }
        }
    }
    //if(arg_a){
    	md_f->Drive(arg_a, duty[0], 100);
    	md_l->Drive(arg_a, duty[1], 100);
    	md_r->Drive(arg_a, duty[2], 100);
    /*}else{
    	md_f->Drive(arg_a, duty[0], 100);
    	md_l->Drive(arg_a, duty[1], 100);
    	md_r->Drive(arg_a, duty[2], 100);
    }*/
}

void Omni_3::Matrix(double speed[3], double duty[3])
{
    double keisu[3][3];
    keisu[0][0] = 1.0;
    keisu[0][1] = 0.0;
    keisu[0][2] = 1.0;
    keisu[1][0] = -1.0 / 2.0;
    keisu[1][1] = -sqrt(3.0) / 2.0;
    keisu[1][2] = 1.0;
    keisu[2][0] = -1.0 / 2.0;
    keisu[2][1] = sqrt(3.0) / 2.0;
    keisu[2][2] = 1.0;

    for(int i = 0; i < 3; i++){
        duty[i] = 0.0;
        for(int j = 0; j < 3; j++){
            duty[i] += keisu[i][j] * speed[j] * rev;
        }
    }
}